
/* *******************************************************************
  A simple Arduino ethernet web server.

  This version:
  - don't uses Arduino String class
  - read parameters (GET and POST)
  - care about non defined pages
  - keep loop tidy
  - handle favicon.ico

  description:
  - en: https://werner.rothschopf.net/202003_arduino_webserver_post_en.htm
  - de: https://werner.rothschopf.net/202001_arduino_webserver_post.htm

  credits:
  idea roughly based on https://stackoverflow.com/questions/14944773/receiving-a-http-post-request-on-arduino by John Harrison

  by noiasca
  2022-02-22  enable output pins with 2 digits, e.g. for MEGA
  2020-11-14  SRAM version (PROGMEM: 14028 / SRAM: 1053)
* *************************************************************** */

#include <SPI.h>
#include <Ethernet.h>

byte mac[] {0x52, 0x64, 0x75, 0x69, 0x6E, 0x6F};         // you can change the MAC and IP addresses to suit your network
IPAddress ip(172, 18, 67, 86);
EthernetServer server(80);                               // Port 80 is HTTP port
const byte ledPin[] {2, 3, 5, 6, 7, 8, 9, A0, A1};       // define the LED pins
//On UNO/NANO don't use 0 (RX), 1(TX), 10 (SS ETH), 11 (MOSI), 12(MISO), 13(CLK). Be carefull with 4(SS SD) if you are using a shield with SD Card.

void printHardwareInfo()
{
  if (Ethernet.hardwareStatus() == EthernetNoHardware) {
    Serial.println(F("Ethernet shield was not found"));
  }
  else if (Ethernet.hardwareStatus() == EthernetW5100) {
    Serial.println(F("W5100 detected"));
  }
  else
  {
    if (Ethernet.hardwareStatus() == EthernetW5200) {
      Serial.println(F("W5200 detected"));
    }
    else if (Ethernet.hardwareStatus() == EthernetW5500) {
      Serial.println(F("W5500 detected"));
    }
    if (Ethernet.linkStatus() == Unknown) {      // cable connection
      Serial.println(F("Link status detection is only available with W5200 and W5500"));
    }
    else if (Ethernet.linkStatus() == LinkON) {
      Serial.println(F("Link status: On"));
    }
    else if (Ethernet.linkStatus() == LinkOFF) {
      Serial.println(F("Link status: Off"));
    }
  }
}

void setup()
{
  Serial.begin(115200);
  Serial.println(F("\nsimple webserver"));
  for (auto &pin : ledPin) {
    pinMode(pin, OUTPUT);    // Set the digital pins ready to write to
  }
  Ethernet.begin(mac, ip);   // Start the Ethernet shield
  printHardwareInfo();
  server.begin();            // Start the webserver
  Serial.print(F("server available at http://"));
  Serial.println(Ethernet.localIP());
}

void loop()
{
  // listen for incoming clients
  checkForClient();
  // put your other main code here, to run repeatedly. avoid any delay()
}
